<?php

namespace Trackeasy\Classes;

// If this file is called directly, abort.
if (!defined('ABSPATH')) {
    exit();
}

use Trackeasy\Interfaces\TE_Trackeasy_Form_Plugin_Integration_Interface;

/**
 * TE_Trackeasy_CF7_Integration class
 */
final class TE_Trackeasy_CF7_Integration implements TE_Trackeasy_Form_Plugin_Integration_Interface
{
    /**
     * @var array|string[]
     * @access public
     * @static
     */
    public static array $allowed_cf7_field_types = [
        'text',
        'email',
        'tel',
        'textarea',
        'url',
        'date',
        'number',
        'select',
        'checkbox',
        'radio',
        'quiz',
        'acceptance',
        'password',
        'hidden'
    ];

    /**
     * TE_Trackeasy_CF7_Integration constructor
     */
    public function __construct()
    {
        $this->te_run_form_plugin_integration();
    }

    /**
     * Integrates Contact Form 7 with TrackEasy.
     *
     * @return void
     * @access public
     */
    public function te_run_form_plugin_integration(): void
    {
        add_action('wpcf7_init', [$this, 'te_wpcf7_add_form_tag_trackeasy']);
        add_action('wpcf7_mail_sent', [$this, 'te_wpcf7_form_submission_handler']);
    }

    /**
     * Adds the TrackEasy campaign ID form tag to Contact Form 7.
     *
     * @return void
     * @access public
     */
    public function te_wpcf7_add_form_tag_trackeasy(): void
    {
        wpcf7_add_form_tag(
            'te_campaignid',
            [$this, 'te_wpcf7_form_tag_handler']
        );
    }

    /**
     * Outputs the TrackEasy campaign ID form tag as a hidden field.
     *
     * @param \WPCF7_FormTag $tag
     * @return string
     * @access public
     */
    public function te_wpcf7_form_tag_handler(\WPCF7_FormTag $tag): string
    {
        $type = $tag->type;
        $values = $tag->values;
        $output = '';

        if ($type === 'te_campaignid' && !empty($values)) {
            $output = '<input class="wpcf7-form-control wpcf7-hidden" type="hidden" name="' . $type . '" value="' . $values[0] . '" />';
        }

        return $output;
    }

    /**
     * Handles the submission of a Contact Form 7 form data to TrackEasy.
     *
     * @param \WPCF7_ContactForm $contact_from
     * @return void
     * @access public
     */
    public function te_wpcf7_form_submission_handler(\WPCF7_ContactForm $contact_form): void
    {
        $submission = \WPCF7_Submission::get_instance();

        if ($submission) {
            $form_field_data = $submission->get_posted_data();

            if (isset($form_field_data['te_campaignid'])) {

                $campaign_id = sanitize_text_field($form_field_data['te_campaignid']);
                $api_key = get_option(TE_Trackeasy_Options::TE_API_KEY);

                if (empty($campaign_id) || empty($api_key)) {

                    error_log(
                        sprintf(
                            '%s Submission Error: The following fields must have values: %s and %s. Contact form ID: %s',
                            'TrackEasy Contact Form 7',
                            'te_campaignid',
                            'te_api_key',
                            $contact_form->id()
                        )
                    );
                } else {

                    if (
                        !TE_Trackeasy_Validator::te_string_is_int_val($campaign_id)
                        || !TE_Trackeasy_Validator::te_string_is_int_val($api_key)
                    ) {

                        error_log(
                            sprintf(
                                '%s Submission Error: The following fields must have integer values: %s and %s. Contact form ID: %s',
                                'TrackEasy Contact Form 7',
                                'te_campaignid',
                                'te_api_key',
                                $contact_form->id()
                            )
                        );
                    } else {

                        $form_tags = array_filter($contact_form->scan_form_tags(), fn ($tag) =>
                            !empty($tag->raw_name) && in_array($tag->basetype, self::$allowed_cf7_field_types));

                        $formatted_form_field_data = [];

                        foreach($form_tags as $form_tag) {
                            $formatted_form_field_data[$form_tag->raw_name] = [
                                'type' => $form_tag->basetype,
                                'value' => $form_field_data[$form_tag->raw_name]
                            ];
                        }

                        TE_Trackeasy_Sanitiser::te_sanitise_field_values($formatted_form_field_data);

                        error_log(sprintf(
                            __('Submitted field data from %s form %s using %s campaign %s: %s', 'trackeasy'),
                            'Contact Form 7',
                            $contact_form->id(),
                            'TrackEasy',
                            $campaign_id,
                            print_r($formatted_form_field_data, true)
                        ));

                        TE_Trackeasy_REST_Client::te_send_request(intval($api_key), intval($campaign_id), $formatted_form_field_data);
                    }
                }
            }
        }
    }
}
